@@ -1,10 +1,8 @@
 import asyncio
-import collections
 import logging
 import secrets
 import falcon
 import rapidjson
-import time
 from falcon import media
 from websockets.exceptions import ConnectionClosedError
 
@@ -16,6 +14,7 @@ from .db import get_storage
 from .rate_limiter import get_rate_limiter
 from . import __version__
 from .config import Config
+from .errors import AuthenticationError
 
 
 class Client:
@@ -35,7 +34,7 @@ class Client:
             return False
         if len(message) < 2:
             return False
-        if message[0] not in ('EVENT', 'REQ', 'CLOSE'):
+        if message[0] not in ('EVENT', 'REQ', 'CLOSE', 'AUTH'):
             return False
         return True
 
@@ -68,6 +67,7 @@ class Client:
         client_id = self.id
         remote_addr = self.remote_addr
 
+        auth_token = {}
         while ws.ready:
             try:
                 message = await ws.receive_media()
@@ -89,13 +89,13 @@ class Client:
                     if self.send_task is None:
                         self.send_task = asyncio.create_task(self.send_subscriptions())
                         await asyncio.sleep(0)
-                    await storage.subscribe(client_id, sub_id, message[2:], self.subscription_queue)
+                    await storage.subscribe(client_id, sub_id, message[2:], self.subscription_queue, auth_token=auth_token)
                 elif command == 'CLOSE':
                     sub_id = str(message[1])
                     await storage.unsubscribe(client_id, sub_id)
                 elif command == 'EVENT':
                     try:
-                        event, result = await storage.add_event(message[1])
+                        event, result = await storage.add_event(message[1], auth_token=auth_token)
                     except Exception as e:
                         LOG.error(str(e))
                         result = False
@@ -106,6 +106,11 @@ class Client:
                         reason = '' if result else 'duplicate: exists'
                         LOG.info("%s added %s from %s", client_id, event.id, event.pubkey)
                     await ws.send_media(['OK', eventid, result, reason])
+                elif command == 'AUTH':
+                    auth_token = await storage.authenticator.authenticate(message[1])
+            except AuthenticationError as e:
+                LOG.warning("Auth error. %s token:%s", str(e), auth_token)
+                await ws.send_media(["NOTICE", str(e)])
             except (falcon.WebSocketDisconnected, ConnectionClosedError):
                 break
             except rapidjson.JSONDecodeError:
@@ -141,16 +146,19 @@ class NostrAPI(BaseResource):
 
     async def on_get(self, req: falcon.Request, resp: falcon.Response):
         if req.accept == 'application/nostr+json':
-            media = {
+            supported_nips = [1, 2, 5, 9, 11, 12, 15, 20, 26, 40]
+            if Config.authentication.get('enabled'):
+                supported_nips.append(42)
+            metadata = {
                 'name': Config.relay_name,
                 'description': Config.relay_description,
                 'pubkey': Config.sysop_pubkey,
                 'contact': Config.sysop_contact,
-                'supported_nips': [1, 2, 5, 9, 11, 12, 15, 20, 26, 40],
+                'supported_nips': supported_nips,
                 'software': 'https://code.pobblelabs.org/fossil/nostr_relay',
                 'version': __version__,
             }
-            resp.media = media
+            resp.media = metadata
         elif Config.redirect_homepage:
             raise falcon.HTTPFound(Config.redirect_homepage)
         else:
@@ -240,19 +248,17 @@ class SetupMiddleware:
 
 
 
-def create_app(conf_file=None):
+def create_app(conf_file=None, storage=None):
     import os
     import os.path
     import logging, logging.config
     from functools import partial
 
-    print(f"Loading configuration from {conf_file}")
-
     Config.load(conf_file)
     if Config.DEBUG:
         print(Config)
 
-    storage = get_storage()
+    storage = storage or get_storage()
     if Config.logging:
         logging.config.dictConfig(Config.logging)
     else:
